@@ -10,15 +10,19 @@
  */
 
 #include <stdio.h>
-#include <io.h>
+#ifdef __CYGWIN__
+#   include <unistd.h>
+#else
+#   include <io.h>
+#endif
 #include <string.h>
 
 int
-main()
-{   
+main(void)
+{
     char buf[1024];
     int n;
-    char *err;
+    const char *err;
 
     while (1) {
 	n = read(0, buf, sizeof(buf));
@@ -32,4 +36,3 @@ main()
 
     return 0;
 }
-	
