@@ -10,12 +10,11 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tclWinFile.c,v 1.95.2.4 2010/04/19 07:40:41 nijtmans Exp $
  */
 
-/* #define _WIN32_WINNT	0x0500 */
-
+#ifndef _WIN64
+#   define _USE_32BIT_TIME_T
+#endif
 #include "tclWinInt.h"
 #include "tclFileSystem.h"
 #include <winioctl.h>
@@ -1282,7 +1281,7 @@ WinIsReserved(
  *	because for NTFS root volumes, the getFileAttributesProc returns a
  *	'hidden' attribute when it should not.
  *
- *	We never make any calss to a 'get attributes' routine here, since we
+ *	We never make any calls to a 'get attributes' routine here, since we
  *	have arranged things so that our caller already knows such
  *	information.
  *
@@ -2266,8 +2265,8 @@ NativeStatMode(
      * positions.
      */
 
-    mode |= (mode & 0x0700) >> 3;
-    mode |= (mode & 0x0700) >> 6;
+    mode |= (mode & (S_IREAD|S_IWRITE|S_IEXEC)) >> 3;
+    mode |= (mode & (S_IREAD|S_IWRITE|S_IEXEC)) >> 6;
     return (unsigned short) mode;
 }
 
